"use strict";
var ui_event_manager_1 = require('../../util/ui-event-manager');
var dom_1 = require('../../util/dom');
var AUTO_SCROLL_MARGIN = 60;
var SCROLL_JUMP = 10;
var ITEM_REORDER_ACTIVE = 'reorder-active';
/**
 * @private
 */
var ItemReorderGesture = (function () {
    function ItemReorderGesture(list) {
        var _this = this;
        this.list = list;
        this.selectedItem = null;
        this.events = new ui_event_manager_1.UIEventManager(false);
        var element = this.list.getNativeElement();
        this.events.pointerEvents(element, function (ev) { return _this.onDragStart(ev); }, function (ev) { return _this.onDragMove(ev); }, function (ev) { return _this.onDragEnd(ev); });
    }
    ItemReorderGesture.prototype.onDragStart = function (ev) {
        var itemEle = ev.target;
        if (itemEle.nodeName !== 'ION-REORDER') {
            return false;
        }
        var item = itemEle['$ionComponent'];
        if (!item) {
            void 0;
            return false;
        }
        ev.preventDefault();
        // Preparing state    
        this.offset = dom_1.pointerCoord(ev);
        this.offset.y += this.list.scrollContent(0);
        this.selectedItem = item;
        this.itemHeight = item.height();
        this.lastToIndex = item.index;
        this.windowHeight = window.innerHeight - AUTO_SCROLL_MARGIN;
        item.setCssClass(ITEM_REORDER_ACTIVE, true);
        return true;
    };
    ItemReorderGesture.prototype.onDragMove = function (ev) {
        var _this = this;
        if (!this.selectedItem) {
            return;
        }
        ev.preventDefault();
        // Get coordinate
        var coord = dom_1.pointerCoord(ev);
        // Scroll if we reach the scroll margins
        var scrollPosition = this.scroll(coord);
        // Update selected item position    
        var ydiff = Math.round(coord.y - this.offset.y + scrollPosition);
        this.selectedItem.setCssStyle(dom_1.CSS.transform, "translateY(" + ydiff + "px)");
        // Only perform hit test if we moved at least 30px from previous position    
        if (Math.abs(coord.y - this.lastYcoord) < 30) {
            return;
        }
        // Hit test
        var overItem = this.itemForCoord(coord);
        if (!overItem) {
            this.emptyZone = true;
            return;
        }
        // Move surrounding items if needed 
        var toIndex = overItem.index;
        if (toIndex !== this.lastToIndex || this.emptyZone) {
            var fromIndex_1 = this.selectedItem.index;
            this.lastToIndex = overItem.index;
            this.lastYcoord = coord.y;
            this.emptyZone = false;
            dom_1.nativeRaf(function () {
                _this.list.reorderMove(fromIndex_1, toIndex, _this.itemHeight);
            });
        }
    };
    ItemReorderGesture.prototype.onDragEnd = function (ev) {
        var _this = this;
        if (!this.selectedItem) {
            return;
        }
        dom_1.nativeRaf(function () {
            var toIndex = _this.lastToIndex;
            var fromIndex = _this.selectedItem.index;
            _this.selectedItem.setCssClass(ITEM_REORDER_ACTIVE, false);
            _this.selectedItem = null;
            _this.list.reorderEmit(fromIndex, toIndex);
        });
    };
    ItemReorderGesture.prototype.itemForCoord = function (coord) {
        var element = document.elementFromPoint(this.offset.x - 100, coord.y);
        if (!element) {
            return null;
        }
        element = dom_1.closest(element, 'ion-item', true);
        if (!element) {
            return null;
        }
        var item = element['$ionComponent'];
        if (!item) {
            void 0;
            return null;
        }
        return item;
    };
    ItemReorderGesture.prototype.scroll = function (coord) {
        var scrollDiff = 0;
        if (coord.y < AUTO_SCROLL_MARGIN) {
            scrollDiff = -SCROLL_JUMP;
        }
        else if (coord.y > this.windowHeight) {
            scrollDiff = SCROLL_JUMP;
        }
        return this.list.scrollContent(scrollDiff);
    };
    /**
     * @private
     */
    ItemReorderGesture.prototype.destroy = function () {
        this.events.unlistenAll();
        this.events = null;
        this.list = null;
    };
    return ItemReorderGesture;
}());
exports.ItemReorderGesture = ItemReorderGesture;
