var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import { UIEventManager } from '../../util/ui-event-manager';
import { closest, pointerCoord, CSS, nativeRaf } from '../../util/dom';
var AUTO_SCROLL_MARGIN = 60;
var SCROLL_JUMP = 10;
var ITEM_REORDER_ACTIVE = 'reorder-active';
/**
 * @private
 */
export var ItemReorderGesture = function () {
    function ItemReorderGesture(list) {
        var _this = this;

        _classCallCheck(this, ItemReorderGesture);

        this.list = list;
        this.selectedItem = null;
        this.events = new UIEventManager(false);
        var element = this.list.getNativeElement();
        this.events.pointerEvents(element, function (ev) {
            return _this.onDragStart(ev);
        }, function (ev) {
            return _this.onDragMove(ev);
        }, function (ev) {
            return _this.onDragEnd(ev);
        });
    }

    _createClass(ItemReorderGesture, [{
        key: 'onDragStart',
        value: function onDragStart(ev) {
            var itemEle = ev.target;
            if (itemEle.nodeName !== 'ION-REORDER') {
                return false;
            }
            var item = itemEle['$ionComponent'];
            if (!item) {
                console.error('item does not contain ion component');
                return false;
            }
            ev.preventDefault();
            // Preparing state   
            this.offset = pointerCoord(ev);
            this.offset.y += this.list.scrollContent(0);
            this.selectedItem = item;
            this.itemHeight = item.height();
            this.lastToIndex = item.index;
            this.windowHeight = window.innerHeight - AUTO_SCROLL_MARGIN;
            item.setCssClass(ITEM_REORDER_ACTIVE, true);
            return true;
        }
    }, {
        key: 'onDragMove',
        value: function onDragMove(ev) {
            var _this2 = this;

            if (!this.selectedItem) {
                return;
            }
            ev.preventDefault();
            // Get coordinate
            var coord = pointerCoord(ev);
            // Scroll if we reach the scroll margins
            var scrollPosition = this.scroll(coord);
            // Update selected item position   
            var ydiff = Math.round(coord.y - this.offset.y + scrollPosition);
            this.selectedItem.setCssStyle(CSS.transform, 'translateY(' + ydiff + 'px)');
            // Only perform hit test if we moved at least 30px from previous position   
            if (Math.abs(coord.y - this.lastYcoord) < 30) {
                return;
            }
            // Hit test
            var overItem = this.itemForCoord(coord);
            if (!overItem) {
                this.emptyZone = true;
                return;
            }
            // Move surrounding items if needed
            var toIndex = overItem.index;
            if (toIndex !== this.lastToIndex || this.emptyZone) {
                (function () {
                    var fromIndex = _this2.selectedItem.index;
                    _this2.lastToIndex = overItem.index;
                    _this2.lastYcoord = coord.y;
                    _this2.emptyZone = false;
                    nativeRaf(function () {
                        _this2.list.reorderMove(fromIndex, toIndex, _this2.itemHeight);
                    });
                })();
            }
        }
    }, {
        key: 'onDragEnd',
        value: function onDragEnd(ev) {
            var _this3 = this;

            if (!this.selectedItem) {
                return;
            }
            nativeRaf(function () {
                var toIndex = _this3.lastToIndex;
                var fromIndex = _this3.selectedItem.index;
                _this3.selectedItem.setCssClass(ITEM_REORDER_ACTIVE, false);
                _this3.selectedItem = null;
                _this3.list.reorderEmit(fromIndex, toIndex);
            });
        }
    }, {
        key: 'itemForCoord',
        value: function itemForCoord(coord) {
            var element = document.elementFromPoint(this.offset.x - 100, coord.y);
            if (!element) {
                return null;
            }
            element = closest(element, 'ion-item', true);
            if (!element) {
                return null;
            }
            var item = element['$ionComponent'];
            if (!item) {
                console.error('item does not have $ionComponent');
                return null;
            }
            return item;
        }
    }, {
        key: 'scroll',
        value: function scroll(coord) {
            var scrollDiff = 0;
            if (coord.y < AUTO_SCROLL_MARGIN) {
                scrollDiff = -SCROLL_JUMP;
            } else if (coord.y > this.windowHeight) {
                scrollDiff = SCROLL_JUMP;
            }
            return this.list.scrollContent(scrollDiff);
        }
        /**
         * @private
         */

    }, {
        key: 'destroy',
        value: function destroy() {
            this.events.unlistenAll();
            this.events = null;
            this.list = null;
        }
    }]);

    return ItemReorderGesture;
}();