var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var __decorate = this && this.__decorate || function (decorators, target, key, desc) {
    var c = arguments.length,
        r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc,
        d;
    if ((typeof Reflect === "undefined" ? "undefined" : _typeof(Reflect)) === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);else for (var i = decorators.length - 1; i >= 0; i--) {
        if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    }return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = this && this.__metadata || function (k, v) {
    if ((typeof Reflect === "undefined" ? "undefined" : _typeof(Reflect)) === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = this && this.__param || function (paramIndex, decorator) {
    return function (target, key) {
        decorator(target, key, paramIndex);
    };
};
import { Attribute, Directive, ElementRef, EventEmitter, Input, NgZone, Optional, Output, Renderer } from '@angular/core';
import { Content } from '../content/content';
import { Ion } from '../ion';
import { isTrueProperty } from '../../util/util';
import { ItemSlidingGesture } from '../item/item-sliding-gesture';
import { ItemReorderGesture } from '../item/item-reorder-gesture';
import { nativeTimeout } from '../../util/dom';
/**
 * The List is a widely used interface element in almost any mobile app,
 * and can include content ranging from basic text all the way to
 * buttons, toggles, icons, and thumbnails.
 *
 * Both the list, which contains items, and the list items themselves
 * can be any HTML element.
 *
 * Using the List and Item components make it easy to support various
 * interaction modes such as swipe to edit, drag to reorder, and
 * removing items.
 *
 * @demo /docs/v2/demos/list/
 * @see {@link /docs/v2/components#lists List Component Docs}
 *
 */
export var List = function (_Ion) {
    _inherits(List, _Ion);

    function List(elementRef, _zone, _content) {
        _classCallCheck(this, List);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(List).call(this, elementRef));

        _this._zone = _zone;
        _this._content = _content;
        _this._enableReorder = false;
        _this._enableSliding = false;
        _this._lastToIndex = -1;
        _this.ionItemReorder = new EventEmitter();
        return _this;
    }
    /**
     * @private
     */


    _createClass(List, [{
        key: "ngOnDestroy",
        value: function ngOnDestroy() {
            this._slidingGesture && this._slidingGesture.destroy();
            this._reorderGesture && this._reorderGesture.destroy();
        }
        /**
         * Enable the sliding items.
         *
         * ```ts
         * import {Component, ViewChild} from '@angular/core';
         * import {List} from 'ionic-angular';
         *
         * @Component({...})
         * export class MyClass {
         *   @ViewChild(List) list: List;
         *
         *   constructor() { }
         *
         *   stopSliding() {
         *     this.list.enableSlidingItems(false);
         *   }
         * }
         * ```
         * @param {boolean} shouldEnable whether the item-sliding should be enabled or not
         */

    }, {
        key: "enableSlidingItems",
        value: function enableSlidingItems(shouldEnable) {
            var _this2 = this;

            if (this._enableSliding === shouldEnable) {
                return;
            }
            this._enableSliding = shouldEnable;
            if (shouldEnable) {
                console.debug('enableSlidingItems');
                nativeTimeout(function () {
                    return _this2._slidingGesture = new ItemSlidingGesture(_this2);
                });
            } else {
                this._slidingGesture && this._slidingGesture.unlisten();
            }
        }
        /**
         * Close the open sliding item.
         *
         * ```ts
         * import {Component, ViewChild} from '@angular/core';
         * import {List} from 'ionic-angular';
         *
         * @Component({...})
         * export class MyClass {
         *   @ViewChild(List) list: List;
         *
         *   constructor() { }
         *
         *   closeItems() {
         *     this.list.closeSlidingItems();
         *   }
         * }
         * ```
         */

    }, {
        key: "closeSlidingItems",
        value: function closeSlidingItems() {
            this._slidingGesture && this._slidingGesture.closeOpened();
        }
        /**
         * @private
         */

    }, {
        key: "reorderEmit",
        value: function reorderEmit(fromIndex, toIndex) {
            var _this3 = this;

            this.reorderReset();
            if (fromIndex !== toIndex) {
                this._zone.run(function () {
                    _this3.ionItemReorder.emit({
                        from: fromIndex,
                        to: toIndex
                    });
                });
            }
        }
        /**
         * @private
         */

    }, {
        key: "scrollContent",
        value: function scrollContent(scroll) {
            var scrollTop = this._content.getScrollTop() + scroll;
            if (scroll !== 0) {
                this._content.scrollTo(0, scrollTop, 0);
            }
            return scrollTop;
        }
        /**
         * @private
         */

    }, {
        key: "reorderReset",
        value: function reorderReset() {
            var children = this.elementRef.nativeElement.children;
            var len = children.length;
            for (var i = 0; i < len; i++) {
                children[i].style.transform = '';
            }
            this._lastToIndex = -1;
        }
        /**
         * @private
         */

    }, {
        key: "reorderMove",
        value: function reorderMove(fromIndex, toIndex, itemHeight) {
            if (this._lastToIndex === -1) {
                this._lastToIndex = fromIndex;
            }
            var lastToIndex = this._lastToIndex;
            this._lastToIndex = toIndex;
            var children = this.elementRef.nativeElement.children;
            if (toIndex >= lastToIndex) {
                for (var i = lastToIndex; i <= toIndex; i++) {
                    if (i !== fromIndex) {
                        children[i].style.transform = i > fromIndex ? "translateY(" + -itemHeight + "px)" : '';
                    }
                }
            }
            if (toIndex <= lastToIndex) {
                for (var i = toIndex; i <= lastToIndex; i++) {
                    if (i !== fromIndex) {
                        children[i].style.transform = i < fromIndex ? "translateY(" + itemHeight + "px)" : '';
                    }
                }
            }
        }
    }, {
        key: "reorder",
        get: function get() {
            return this._enableReorder;
        },
        set: function set(val) {
            var _this4 = this;

            var enabled = isTrueProperty(val);
            if (this._enableReorder === enabled) {
                return;
            }
            this._enableReorder = enabled;
            if (enabled) {
                console.debug('enableReorderItems');
                nativeTimeout(function () {
                    return _this4._reorderGesture = new ItemReorderGesture(_this4);
                });
            } else {
                this._reorderGesture && this._reorderGesture.destroy();
            }
        }
    }]);

    return List;
}(Ion);
__decorate([Output(), __metadata('design:type', typeof (_a = typeof EventEmitter !== 'undefined' && EventEmitter) === 'function' && _a || Object)], List.prototype, "ionItemReorder", void 0);
__decorate([Input(), __metadata('design:type', Boolean)], List.prototype, "reorder", null);
List = __decorate([Directive({
    selector: 'ion-list',
    host: {
        '[class.reorder-enabled]': '_enableReorder'
    }
}), __param(2, Optional()), __metadata('design:paramtypes', [typeof (_b = typeof ElementRef !== 'undefined' && ElementRef) === 'function' && _b || Object, typeof (_c = typeof NgZone !== 'undefined' && NgZone) === 'function' && _c || Object, typeof (_d = typeof Content !== 'undefined' && Content) === 'function' && _d || Object])], List);
/**
 * @private
 */
export var ListHeader = function () {
    function ListHeader(_renderer, _elementRef, _id) {
        _classCallCheck(this, ListHeader);

        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._id = _id;
    }

    _createClass(ListHeader, [{
        key: "id",
        get: function get() {
            return this._id;
        },
        set: function set(val) {
            this._id = val;
            this._renderer.setElementAttribute(this._elementRef.nativeElement, 'id', val);
        }
    }]);

    return ListHeader;
}();
ListHeader = __decorate([Directive({
    selector: 'ion-list-header'
}), __param(2, Attribute('id')), __metadata('design:paramtypes', [typeof (_e = typeof Renderer !== 'undefined' && Renderer) === 'function' && _e || Object, typeof (_f = typeof ElementRef !== 'undefined' && ElementRef) === 'function' && _f || Object, String])], ListHeader);
var _a, _b, _c, _d, _e, _f;